/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DATABASE_H__
#define __TK_DATABASE_H__

#define GETTEXT_PACKAGE "TkResolver"

#include <stdexcept>

#include <odbc++/preparedstatement.h>

#include "libtkdb/database_types.h"
#include "libtkdb/value.h"
#include "libtkcommon/tkstring.h"
#include "libtkrandom/random_types.h"
#include "libtkresolver/types.h"

#define EXPERIMENT_GROUP "Test group"

namespace Tk {
  namespace Db {
    
    typedef std::pair<Code, Tk::String> IntentInfo;
    typedef std::list<IntentInfo> IntentInfoList;

    class UnknownType : public std::logic_error
    {
    private:
      Tk::String _type_name;
      Tk::String _domain;
    public:
      UnknownType(const Tk::String &type_name, const Tk::String &domain = ""); 
      virtual ~UnknownType() throw ();
      const Tk::String &get_type_name() const;
    };
    
    class PropertyNotSet : public std::runtime_error
    {
    public:
      PropertyNotSet(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name);
    };
    
    
    class Database
      {
      private:
	typedef std::map<Tk::String, Type> FieldInfo;
	typedef std::map<Tk::String, FieldInfo> TableInfo;
	
	odbc::Connection    * _connection;
	
	TableInfo           _table_info;
	
	void get_table_info(const Tk::String &name);
	void check_table_info(const Tk::String &name);
	
	Id insert_property(const Id &property_type_id);
	Id insert_property_value (const Id &property_id);
	void insert_property_value_data (const Id &property_value_id, const Tk::String &table_name, Data::Value &value);
	void update_property_value_data (const Id &property_value_id, const Tk::String &table_name, Data::Value &value);
	void insert_domain_property(const Tk::String &domain, const Id &domain_id, const Id &property_id);
	
	
	Id insert_experiment(const Id &location_id);
	
	void experiment_insert_operators(const Id &experiment_id, const Ids &operators);
	void experiment_insert_result(const Id &experiment_id, const Result &result);
	
      public:
	Database (const Tk::String &data_source_name = Tk::String("TkResolver++"),
		  const Tk::String &username = Tk::String("psi"),
		  const Tk::String &password = Tk::String("isp"));
	~Database();

	Statement  * new_statement();
	void         execute_statement(const Tk::String &sql);
	DataSet    * execute_query(const Tk::String &sql);
	PStatement * prepare_statement(const Tk::String &sql);
	Ids       get_ids(PStatement &stmt);
	
	Type         get_column_type(const Tk::String &table, const Tk::String &column);
	DataSet *    get_table_data(const Tk::String &table, const Tk::StringList &columns);
	PStatement * get_table_insert_statement(const Tk::String &table, const Tk::StringList &columns);
	Id           get_last_insert_id();

	
	void property_type_get_details (const Tk::String &property_domain,
					const Tk::String &property_name,
					Id *property_type_id = 0,
					Tk::String *property_value_type_name = 0,
					Tk::String *property_value_table_name = 0);
	
	Id             lookup_location(const Tk::String &location_name);
	Id             choose_location();
	Id             choose_operator();
	Code           choose_intent(unsigned trace_count, const Tk::String &algorithm_spec, bool test);
	IntentInfoList get_intent_options(unsigned trace_count);
	
	Id             create_entity(const Tk::String &domain);
	Ids            get_entities(const Tk::String &domain);
	Id             find_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name);
	Data::Value    get_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name);
	Id             set_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name, Data::Value &value);
	Data::TypeMap  get_domain_property_types(const Tk::String &domain);
	Data::ValueMap get_properties(const Tk::String &domain, const Id &entity_id);
	DataSet *      get_results(const Id &experiment);
	
	Id insert_experiment(const Id &location_id,
			     const Ids &operators,
			     Data::ValueMap &properties,
			     const Result &result);
	
    };

  }; /* namespace Db */
}; /* namespace Tk */

#endif /* __TK_DATABASE_H__ */
