/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_VALUE_H__
#define __TK_DB_VALUE_H__

#include <boost/variant/variant.hpp>
#include <boost/variant/get.hpp>
#include <odbc++/types.h>

#include "libtkcommon/tkvalue.h"
#include "libtkcommon/numerical-types.h"
#include "libtkdb/database_types.h"
#include "libtkdb/bytes.h"

namespace Tk {
  namespace Db {
    namespace Data {
      
      typedef enum {
	Boolean,
	Integer,
	Unsigned,
	Double,
	String,
	Timestamp,
	Date,
	Blob
      } Type;
      
      typedef std::map<Tk::String, Type> TypeMap;
      
      typedef boost::variant<
	bool,
	int,
	unsigned,
	double,
	Tk::String,
	Tk::Db::DateTime,
	Tk::Db::Date,
	Tk::Db::Bytes
	> datum;
    
      typedef Tk::Value<datum> Datum;
    
      class Value
	{
	private:
	  Type             _type;
	  Datum            _datum;
	  
	  static int type_to_odbc_type(Type type);
	  static Type odbc_type_to_type(int odbc_type);
	  
	  static Type type_of(const bool *);
	  static Type type_of(const int *);
	  static Type type_of(const unsigned *);
	  static Type type_of(const double *);
	  static Type type_of(const Tk::String *);
	  static Type type_of(const Tk::Db::DateTime *);
	  static Type type_of(const Tk::Db::Date *);
	  static Type type_of(const Tk::Db::Bytes *);
	  
	public:
	  static Type get_type(const Tk::String &property_value_type_name);
	  
	  template <typename T>
	  Value(const Tk::Value<T> &v)
	    : _type( type_of(v.get_value_ptr()) )
	  {
	    if (v.is_valid())
	      _datum = v.get_value();
	  }
	  
	  template <typename T>
	  Value(const T &v)
	    : _type( type_of(&v) ), _datum(v) {}
	  
	  Value(Type type, const Datum &datum = Datum());
	  Value(DataSet &result, int index);
	  
	  Type get_type() const;
	  const Datum &get_datum() const;
	  
	  template <typename T>
	  Tk::Value<T> get_value() const
	  { 
	    return _datum.is_null()
	      ? (Tk::Value<T>())
	      : (Tk::Value<T>(boost::get<T>(_datum.get_value())));
	  }
	  
	  void set_parameter(PStatement *stmt, unsigned index);
	};
      
      typedef std::auto_ptr<Value> ValuePtr;
    
      class Values : public std::list<Value>
      {
      public:
	Values();	
	void append(const Values &values);
	void set_parameters(PStatement *stmt, unsigned start_index = 1);
      };
      
      typedef std::map<Tk::String, Value> ValueMap;
      
    }; /* namespace Data */
  }; /* namespace Db */
}; /* namespace Tk */


#endif /* __TK_DB_VALUE_H__ */
