/* -*- c++ -*-
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh> for tkresolver
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 *
 */

template <typename T>
Complex::Number
log(std::complex<T> op)
{
  return Complex::Number( Real::log( static_cast<Real::Number>(op.real()*op.real() + op.imag()*op.imag()) ) / 2,
			  Real::atan2( static_cast<Real::Number>(op.imag()), static_cast<Real::Number>(op.real()) )
			  );
}

template <typename T>
Complex::Number
exp(std::complex<T> op)
{
  return
    Complex::Number( Real::cos( static_cast<Real::Number>(op.imag()) ),
		     Real::sin( static_cast<Real::Number>(op.imag()) )
		     )
    * Real::exp( static_cast<Real::Number>(op.real()) );
}

template <typename T>
Complex::Number
sin(std::complex<T> op)
{
  Real::Number ex( Real::exp(op.imag()) );
  Real::Number sx( Real::sin(op.real()) );
  Real::Number cx( Real::cos(op.real()) );
  return Complex::Number(sx*(ex+1 / ex) / 2,
			 cx*(ex-1 / ex) / 2);
}

