/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdexcept>
#include <boost/assign/list_of.hpp>

#include "libtkcommon/config.h"
#include "libtkcommon/debug.h"
#include "retriever_registry.h"
#include "pseudo_random.h"
#ifdef HAVE_USB
#include "alea.h"
#endif /* HAVE_USB */
#ifdef HAVE_CURLPP
#include "hotbits.h"
#endif /* HAVE_CURLPP */

using namespace Tk;
using namespace Random;

const RetrieverTypeMap RetrieverRegistry::_registry =
boost::assign::map_list_of
(String("pseudo"),    RetrieverDetails(static_cast<RetrieverType *>(new PseudoRandomType()), "Pseudo-random number generator"))

#ifdef HAVE_USB
  (String("alea"),    RetrieverDetails(static_cast<RetrieverType *>(new AleaType()), "Araneus Alea I hardware RNG"))
#endif /* HAVE_USB */

#ifdef HAVE_CURLPP
  (String("hotbits"), RetrieverDetails(static_cast<RetrieverType *>(new HotBitsType()), "HotBits Internet server"))
#endif /* HAVE_CURLPP */
   ;


const RetrieverDetails &
RetrieverRegistry:: get_retriever_details(const String &type)
{
  RetrieverTypeMap::const_iterator iter(_registry.find(type));
  
  if (iter == _registry.end())
    throw std::invalid_argument("unknown retriever type");
  
  return iter->second;
}

StringList
RetrieverRegistry:: get_names()
{
  StringList list;

  for (RetrieverTypeMap::const_iterator iter = _registry.begin();
       iter != _registry.end();
       ++iter)
    {
      list.push_back(iter->first);
    }

  return list;
}



RetrieverType *
RetrieverRegistry:: get_retriever(const String &type)
{
  return get_retriever_details(type).first;
}


unsigned
RetrieverRegistry:: get_preferred_block_size(const String &type)
{
  return get_retriever(type)->get_preferred_block_size();
}

Retriever *
RetrieverRegistry:: new_retriever (const String &type, unsigned count)
{
  return get_retriever(type)->new_retriever(count);
}

bool
RetrieverRegistry:: is_true_randomness(const String &type)
{
  return get_retriever(type)->is_true_randomness();
}

const String &
RetrieverRegistry:: get_retriever_description(const String &name)
{
  return get_retriever_details(name).second;
}

/* EOF */
